# Filename: clr.R
# composite linear regression
#
# Command line arguments:
#    R --no-restore --no-save --args file1,file2 < clr.R > clr.Rout
#
# file1 is the name of the file to read data.
# file2 is the name of the file to save R image.
#author: Jelena Bradic and Weiwei Wang

source("scad-lla.R")
source("penHuber.R")
source("penCQR.R")


# parse command line arguments
args <- commandArgs();
files <- (strsplit(args[length(args)],","))[[1]];

load(files[1]);
n <- 100;
sets <- 100;
tau <- (1:5)/6;

b1p <- b2p <- b12p <- bcqrp <- bwcqrp <- merr <- NULL;
w12 <- wcqr <- NULL;

for(i in 1:sets)
{
  print(i);
  x <- xa[n*(i-1)+(1:n),];
  y <- ya[n*(i-1)+(1:n)];

  newx <- xa[n*(i-1+sets)+(1:n),];
  newy <- ya[n*(i-1+sets)+(1:n)];
  
  # centering data
  x <- scale(x, TRUE, FALSE)
  y <- drop(y - mean(y))

  newx <- scale(newx, TRUE, FALSE)
  newy <- drop(newy - mean(newy))

  # penalized L2
  f2p <- scad.lla(y,x,type="test",newy=newy,newx=newx,s1=1);
  b2p <- rbind(b2p, f2p$beta);
  
  # penalized l1
  f1p <- suppressWarnings(penCQR(y,x,tau=0.5,type="test",newy=newy,newx=newx,s1=1));
  b1p <- rbind(b1p, f1p$betas);

  # penalized L1 and L2
  r <- y-x%*%f2p$beta;
  f12p <- penHuber(y,x,knots=quantile(abs(r),c(0.1,1)), beta0=f2p$beta,type="test",newy=newy,newx=newx);
  b12p <- rbind(b12p,f12p$beta);
  w12 <- rbind(w12,f12p$weight);
  
  
  # penalized CQR
  fcqrp <- suppressWarnings(penCQR(y,x,weight=rep(1,length(tau)), tau=tau,type="test",newy=newy,newx=newx,s1=1,beta0=f1p$beta))
  bcqrp <- rbind(bcqrp, fcqrp$betas);

  fwcqrp <- suppressWarnings(penCQR(y,x,tau=tau,type="test",newy=newy,newx=newx,s1=1,beta0=f1p$beta));
  bwcqrp <- rbind(bwcqrp, fwcqrp$betas);
  wcqr <- rbind(wcqr,fwcqrp$weight);

  merr <- rbind(merr, apply(( x %*% (cbind(f1p$beta,f2p$beta,f12p$beta,fcqrp$beta,fwcqrp$beta) - beta.true))^2,2,mean))
}

t2 <- proc.time();
save.image(file=files[2]);
