expandmatrix<-function(A,B){
  n1<-dim(A)[1]
  p1<-dim(A)[2]
  n2<-dim(B)[1]
  p2<-dim(B)[2]
  A<-cbind(A,matrix(0,n1,p2))
  B<-cbind(matrix(0,n2,p1),B)
  return(rbind(A,B))
}

# adaptation of Zou's algorithm

pcqr<-function(y,x,tau,lambda=0,ws){
  # Tau is a vector of quantiles
  # lambda is the penalization parameter in adaptive lasso
  # ws is the weights in adaptive lasso penalty
  require("lpSolve") 
  call <- match.call()
  K<-length(tau)
  p<-dim(x)[2] 
  n<-length(y)  
  if(missing(ws)){ws<-rep(1,p)}
  cvec<-c(rep(c(0,0,rep(1,n)),K),lambda*ws,lambda*ws)/(K*n)
  bvec<-c(tau[1]*y,(tau[1]-1)*y) 
  A1<-cbind(rep(1,n),rep(-1,n))
  A2<-rbind(diag(n),diag(n))
  Aleft<-cbind(rbind(tau[1]*A1,(tau[1]-1)*A1),A2)   
  Aright<-rbind(tau[1]*cbind(x,-x),(tau[1]-1)*cbind(x,-x))     
  if(K>=2){
  for(j in 2:K){
  Aleft<-expandmatrix(Aleft,cbind(rbind(tau[j]*A1,(tau[j]-1)*A1),A2) )
  Aright<-rbind(Aright,rbind(tau[j]*cbind(x,-x),(tau[j]-1)*cbind(x,-x)))
  bvec<-c(bvec,c(tau[j]*y,(tau[j]-1)*y) )
  }
  }
  Amat<-cbind(Aleft,Aright)
  require("boot")
  #lp.out<-lp(direction="min",objective.in=cvec,const.mat=Amat,const.dir=rep(">=",length(bvec)),const.rhs=bvec)  
  simplex.out<-simplex(cvec, A2=Amat, b2=bvec,n.iter=1);
  Beta<-simpex.out$soln
#  Beta<-lp.out$solution
  btau<-tau
  for (j in 1:K){
     a6<-1+(j-1)*(2+n)
     a7<-j*(2+n)
     bbb<-Beta[(a6:a7)]
     btau[j]<-bbb[1]-bbb[2]
     }
  betas1<-rev(rev(Beta)[1:(2*p)]) 
  betas<-betas1[(1:p)]-betas1[((1+p):(2*p))]
       
object <- list(call = call, tau=tau, lambda=lambda, ws=ws, betas=betas, btau=btau)
object
   
}


