# Weighted L1-L2 regression

require(glmnet)
require(quadprog)

scad <- function(lambda,beta,a=3.7)
{
  theta <- abs(beta);
  lambda*(theta<=lambda)+(a*lambda-theta)*(a*lambda>theta)/(a-1)*(theta>lambda);
}

cv.scad1step <- function(x,y,lambda,beta0,K=5,plot=TRUE)
{
  n <- length(y);
  p <- ncol(x);
  
  all.folds <- split(sample(1:n), rep(1:K,length=n));
  residmat <- matrix(0, length(lambda), K)
  for (i in seq(K)) {
    omit <- all.folds[[i]]

    for(j in 1:length(lambda)){
      fit <- glmnet(x[-omit,], y[-omit],family="gaussian", standardize=F,
                     lambda=1,penalty.factor=scad(lambda[j], beta0));
      residmat[j, i] <- mean((y[omit]-x[omit,]%*%fit$beta)^2)
    }
  }
  cv <- apply(residmat, 1, mean)
  cv.error <- sqrt(apply(residmat, 1, var)/K)
  if(plot){plot(cv~lambda)}
  object <- list(lambda = lambda, cv = cv, cv.error = cv.error)
}

weightL1L2 <- function(r,sigma2)
  {
    n <- length(r);
    b5 <- median(r);
    f0 <- density(r,from=b5,to=b5,n=1,bw="nrd")$y;
    B=(sum(r[r>=b5]) - sum(r[r<b5]))/n;

    weight.QP <- solve.QP(Dmat=matrix(c(1,2*B,2*B,4*sigma2),ncol=2),dvec=rep(0,2),Amat=cbind(c(f0,1),c(1,0),c(0,1)),bvec=c(1,0,0),meq=1);

    wQP <- weight.QP$solution/sum(weight.QP$solution);

    wQP
  }
