# File: wlasso.R
#
# Description: weighted lasso, compute:
#
#              argmin sum_{i=1}^n (y_i-x_i * beta)^2 + n * sum_{j=1}^p lambda_j * |beta_j|
#
#           lambda_j are given, 
#
# Arguments: x -- n by p matrix
#            y -- n by 1 vector
#            penalty -- p by 1 vector, penalty on each beta (i.e. lambda_j)

#
# Assume x and y are centered and x are normalized
# Author: Jelena Bradic and Weiwei Wang

require(lars)

# File: wlasso.R
#
# Description: weighted lasso, compute:
#
#              argmin 1/2*sum_{i=1}^n (y_i-x_i * beta)^2 +  sum_{j=1}^p lambda_j * |beta_j|
#
#           lambda_j are given, 
#
# Arguments: x -- n by p matrix
#            y -- n by 1 vector
#            penalty -- p by 1 vector, penalty on each beta (i.e. lambda_j)

#
# Assume x and y are centered and x are normalized
# Author: Weiwei Wang & Jelena Bradic

wlasso <- function(x,y,penalty=rep(1,ncol(x)))
{
  n <- nrow(x);
  p <- ncol(x);
  
  # find out which penalties are exactly zero;
  setU <- which(penalty<1e-6);
  setV <- setdiff(1:p, setU);


  if (length(setV)==0) # set V is empty, all penalty equal to zero
  {
  	cat("nula penalty")
    #fit <- lm(y~x-1);
    #beta <- fit$coef;
    f1 <- lars(x,y,type="lasso",use.Gram=FALSE,intercept=FALSE)
    cat("test")
  	
    cv1 <- cv.lars(x,y,type="lasso",plot.it=FALSE);
	beta <- predict(f1, s=cv1$fraction[which.min(cv1$cv.error)],type="coefficients",mode="fraction")$coef
	
 

     #fit <- lars(x,y,normalize=F,use.Gram=FALSE,intercept=F,type="lasso",trace=F);
     #beta <- predict.lars(fit,s=1,type="coefficients",mode=c("lambda"))$coef;
      }
  else
  {
    if (length(setU)==0)
    {
    	cat("setU 
    		")
    	#print(penalty)
      x1 <- scale(x,FALSE,penalty);
      #print(y)
      
      f1 <- lars(x1,y,type="lasso",use.Gram=FALSE,intercept=FALSE)
      
	  cv1 <- cv.lars(x1,y,type="lasso",use.Gram=FALSE,plot.it=FALSE);
	  
      b1 <- predict(f1, s=cv1$fraction[which.min(cv1$cv.error)],type="coefficients",mode="fraction")$coef 

      
      #fit <- lars(x1,y,normalize=F,use.Gram=FALSE,intercept=F,type="lasso",trace=F);
      #beta <- predict.lars(fit,s=1,type="coefficients",mode=c("lambda"))$coef;
      beta <- b1/penalty;
      #print(beta)
          }
    else
    {
    	cat("else 
    	")
       Xu <- x[,setU];
      Hu <- Xu %*% solve(t(Xu) %*% Xu) %*% t(Xu);
      y2 <- y-Hu%*%y;
      Xv <- scale(x[,setV], FALSE, penalty[setV]);
      Xv2 <- Xv-Hu%*%Xv;
      
      f1 <- lars(Xv2,y2,type="lasso",use.Gram=FALSE,intercept=FALSE)
	  cv1 <- cv.lars(Xv2,y2,type="lasso",plot.it=FALSE);
	  betaV <- predict(f1, s=cv1$fraction[which.min(cv1$cv.error)],type="coefficients",mode="fraction")$coef

      
      
      #fit <- lars(Xv2,y2,normalize=F,intercept=FALSE,type="lasso",trace=F,use.Gram=F);
      #betaV <- predict.lars(fit,s=1,type="coefficients",mode=c("lambda"))$coef;
      
      
      betaU <- solve(t(Xu)%*%Xu)%*%t(Xu)%*%(y-Xv%*%betaV);
      
      
      beta <- rep(0,p);
      beta[setU] <- betaU;
      beta[setV] <- betaV/penalty[setV];
    }
  }  

#  list(beta=beta,fit=fit);
  beta
}
